@@ -22,14 +22,14 @@
       +-----------------------+  ;;  | |         |,"
          /_)______________(_/  //'   | +---------+
     ___________________________/___  `,
-   /  oooooooooooooooo  .o.  oooo /,   \,"-----------
-  / ==ooooooooooooooo==.o.  ooo= //   ,`\--{)B     ,"
+   /  oooooooooooooooo  .o.  oooo /,   `,"-----------
+  / ==ooooooooooooooo==.o.  ooo= //   ,``--{)B     ,"
  /_==__==========__==_ooo__ooo=_/'   /___________,"
 """
 
+from . import metadata
 from . import utils
 from . import decorator
-from ._logging import LogHandler
-from ._tools import ToolContainer, ToolExecError, Tool
+from ._tools import Tools, Tool
 from ._config import Config
 from ._environ import BaseEnviron, environ
