@@ -22,9 +22,23 @@
       +-----------------------+  ;;  | |         |,"
          /_)______________(_/  //'   | +---------+
     ___________________________/___  `,
-   /  oooooooooooooooo  .o.  oooo /,   \,"-----------
-  / ==ooooooooooooooo==.o.  ooo= //   ,`\--{)B     ,"
+   /  oooooooooooooooo  .o.  oooo /,   `,"-----------
+  / ==ooooooooooooooo==.o.  ooo= //   ,``--{)B     ,"
  /_==__==========__==_ooo__ooo=_/'   /___________,"
 """
 
-from ._command import BaseCommand, walk_commands
+from . import argparse
+
+from .command import \
+    BaseCommand, BaseCommandGroup, CommandError, \
+    SubCommand, SubCommandGroup, SubCommandWrapper, \
+    subcommand, subcommand_argument, SubCommandError, NotFoundSubCommand, \
+    iter_module_commands, iter_entry_point_commands, \
+    CommandMain, CommandParser
+
+from .update import UpdateCommand, PypiUpdater, DevelopUpdater, GitUpdater
+
+from .mobile import \
+    DeviceCommandMixin, \
+    AndroidCommandMixin, AndroidCommand, AndroidNamespace, \
+    IOSCommandMixin, IOSCommand, IOSNamespace
