@@ -22,20 +22,18 @@
       +-----------------------+  ;;  | |         |,"
          /_)______________(_/  //'   | +---------+
     ___________________________/___  `,
-   /  oooooooooooooooo  .o.  oooo /,   \,"-----------
-  / ==ooooooooooooooo==.o.  ooo= //   ,`\--{)B     ,"
+   /  oooooooooooooooo  .o.  oooo /,   `,"-----------
+  / ==ooooooooooooooo==.o.  ooo= //   ,``--{)B     ,"
  /_==__==========__==_ooo__ooo=_/'   /___________,"
 """
-from argparse import ArgumentParser
 from typing import Optional
 
-from linktools.android import Adb
-from linktools.cli.android import AndroidCommand
+from linktools.cli import CommandParser, CommandMain, AndroidCommand, AndroidNamespace
 
 
 class Command(AndroidCommand):
     """
-    Adb that supports multiple devices
+    Manage multiple Android devices effortlessly with adb commands
     """
 
     _GENERAL_COMMANDS = [
@@ -45,26 +43,30 @@ class Command(AndroidCommand):
         "connect",
         "disconnect",
         "keygen",
-        "wait-for-",
+        # "wait-for-",
         "start-server",
         "kill-server",
         "reconnect",
+        "attach",
+        "detach",
     ]
 
-    def init_arguments(self, parser: ArgumentParser) -> None:
-        parser.add_argument("adb_args", nargs="...", help="adb args")
+    @property
+    def main(self) -> CommandMain:
+        return CommandMain(self, show_log_time=False, show_log_level=False)
 
-    def run(self, args: [str]) -> Optional[int]:
-        args, extra = self.parse_known_args(args)
+    def init_arguments(self, parser: CommandParser) -> None:
+        parser.add_argument("adb_args", nargs="...", metavar="args", help="adb args")
 
-        adb_args = [*extra, *args.adb_args]
-        if not extra:
-            if args.adb_args and args.adb_args[0] not in self._GENERAL_COMMANDS:
-                device = args.parse_device()
-                process = device.popen(*adb_args, capture_output=False)
-                return process.call()
+    def run(self, args: AndroidNamespace) -> Optional[int]:
+        adb_args = args.adb_args
+        if adb_args and adb_args[0] not in self._GENERAL_COMMANDS and not adb_args[0].startswith("wait-for-"):
+            device = args.device_selector.select()
+            process = device.popen(*adb_args, capture_output=False)
+            return process.call()
 
-        process = Adb.popen(*adb_args, capture_output=False)
+        adb = args.device_selector.bridge
+        process = adb.popen(*adb_args, capture_output=False)
         return process.call()
 
 
