@@ -22,27 +22,26 @@
       +-----------------------+  ;;  | |         |,"
          /_)______________(_/  //'   | +---------+
     ___________________________/___  `,
-   /  oooooooooooooooo  .o.  oooo /,   \,"-----------
-  / ==ooooooooooooooo==.o.  ooo= //   ,`\--{)B     ,"
+   /  oooooooooooooooo  .o.  oooo /,   `,"-----------
+  / ==ooooooooooooooo==.o.  ooo= //   ,``--{)B     ,"
  /_==__==========__==_ooo__ooo=_/'   /___________,"
 """
-from argparse import ArgumentParser
 from typing import Optional
 
 from linktools import utils
-from linktools.cli.android import AndroidCommand
+from linktools.cli import CommandParser, AndroidCommand, AndroidNamespace
 
 
 class Command(AndroidCommand):
     """
-    Debug app by jdb
+    Debug Android apps effectively using the Java Debugger (jdb)
     """
 
     @property
     def _description(self) -> str:
         return "debugger"
 
-    def init_arguments(self, parser: ArgumentParser) -> None:
+    def init_arguments(self, parser: CommandParser) -> None:
         parser.add_argument('package', action='store', default=None,
                             help='regular expression')
         parser.add_argument('activity', action='store', default=None,
@@ -50,9 +49,8 @@ class Command(AndroidCommand):
         parser.add_argument('-p', '--port', action='store', type=int, default=8701,
                             help='fetch all apps')
 
-    def run(self, args: [str]) -> Optional[int]:
-        args = self.parse_args(args)
-        device = args.parse_device()
+    def run(self, args: AndroidNamespace) -> Optional[int]:
+        device = args.device_selector.select()
 
         device.shell("am", "force-stop", args.package, log_output=True)
         device.shell("am", "start", "-D", "-n", "{}/{}".format(args.package, args.activity), log_output=True)
@@ -61,8 +59,8 @@ class Command(AndroidCommand):
         with device.forward(f"tcp:{args.port}", f"jdwp:{pid}"):
             data = input("jdb connect? [Y/n]: ").strip()
             if data in ["", "Y", "y"]:
-                process = utils.Popen("jdb", "-connect",
-                                      "com.sun.jdi.SocketAttach:hostname=127.0.0.1,port={}".format(args.port))
+                process = utils.popen(
+                    "jdb", "-connect", f"com.sun.jdi.SocketAttach:hostname=127.0.0.1,port={args.port}")
                 return process.call()
 
 
