@@ -22,15 +22,14 @@
       +-----------------------+  ;;  | |         |,"
          /_)______________(_/  //'   | +---------+
     ___________________________/___  `,
-   /  oooooooooooooooo  .o.  oooo /,   \,"-----------
-  / ==ooooooooooooooo==.o.  ooo= //   ,`\--{)B     ,"
+   /  oooooooooooooooo  .o.  oooo /,   `,"-----------
+  / ==ooooooooooooooo==.o.  ooo= //   ,``--{)B     ,"
  /_==__==========__==_ooo__ooo=_/'   /___________,"
 """
-from argparse import ArgumentParser
 from typing import Optional
 
 from linktools import environ
-from linktools.cli.android import AndroidCommand
+from linktools.cli import CommandParser, AndroidCommand, AndroidNamespace
 
 props = (
     "ro.product.manufacturer",
@@ -73,15 +72,14 @@ cmds = (
 
 class Command(AndroidCommand):
     """
-    Fetch device information
+    Collect detailed device information
     """
 
-    def init_arguments(self, parser: ArgumentParser) -> None:
+    def init_arguments(self, parser: CommandParser) -> None:
         parser.add_argument('agent_args', nargs='...', help="agent args")
 
-    def run(self, args: [str]) -> Optional[int]:
-        args = self.parse_args(args)
-        device = args.parse_device()
+    def run(self, args: AndroidNamespace) -> Optional[int]:
+        device = args.device_selector.select()
 
         environ.logger.info(f"Property", style="red")
         for prop in props:
