@@ -22,24 +22,29 @@
       +-----------------------+  ;;  | |         |,"
          /_)______________(_/  //'   | +---------+
     ___________________________/___  `,
-   /  oooooooooooooooo  .o.  oooo /,   \,"-----------
-  / ==ooooooooooooooo==.o.  ooo= //   ,`\--{)B     ,"
+   /  oooooooooooooooo  .o.  oooo /,   `,"-----------
+  / ==ooooooooooooooo==.o.  ooo= //   ,``--{)B     ,"
  /_==__==========__==_ooo__ooo=_/'   /___________,"
 """
-from argparse import ArgumentParser
-from typing import Optional
+from typing import Optional, List, Type
 
 from linktools import utils, environ
-from linktools.cli.android import AndroidCommand
-from linktools.frida.android import AndroidFridaServer
+from linktools.cli import CommandParser, CommandError, AndroidCommand, AndroidNamespace
+from linktools.cli.argparse import range_type, BooleanOptionalAction
+from linktools.frida import FridaAndroidServer
+from linktools.types import DownloadError
 
 
 class Command(AndroidCommand):
     """
-    Easy to use objection (require Android device rooted)
+    Simplify security testing with Objection on rooted Android devices
     """
 
-    def init_arguments(self, parser: ArgumentParser) -> None:
+    @property
+    def known_errors(self) -> List[Type[BaseException]]:
+        return super().known_errors + [DownloadError]
+
+    def init_arguments(self, parser: CommandParser) -> None:
         parser.add_argument("-p", "--package", action="store", default=None,
                             help="target package (default: frontmost application)")
         parser.add_argument("-s", "--startup-command", action="append", default=[],
@@ -52,20 +57,34 @@ class Command(AndroidCommand):
         parser.add_argument("-P", "--plugin-folder", action="store", default=environ.get_asset_path("objection"),
                             help="The folder to load plugins from.")
 
+        parser.add_argument("--serve", action=BooleanOptionalAction, default=True,
+                            help="serve frida server (default: true)")
+        parser.add_argument("--local-port", metavar="PORT", action="store", dest="local_port",
+                            type=range_type(1, 65536), default=None,
+                            help="local frida port (default: unused port)")
+        parser.add_argument("--remote-port", metavar="PORT", action="store", dest="remote_port",
+                            type=range_type(1, 65536), default=27042,
+                            help="remote frida port (default: 27042)")
+
         parser.add_argument("--redirect-address", metavar="ADDRESS", action="store", dest="redirect_address",
                             type=str,
                             help="redirect traffic to target address (default: localhost)")
         parser.add_argument("--redirect-port", metavar="PORT", action="store", dest="redirect_port",
-                            type=utils.range_type(1, 65536),
+                            type=range_type(1, 65536),
                             help="redirect traffic to target port (default: 8080)")
 
-    def run(self, args: [str]) -> Optional[int]:
-        args = self.parse_args(args)
-        device = args.parse_device()
+    def run(self, args: AndroidNamespace) -> Optional[int]:
+        device = args.device_selector.select()
 
-        with AndroidFridaServer(device=device, local_port=utils.pick_unused_port()) as server:
+        server = FridaAndroidServer(
+            device=device,
+            local_port=args.local_port or utils.get_free_port(),
+            remote_port=args.remote_port,
+            serve=args.serve
+        )
 
-            objection_args = ["objection"]
+        with server:
+            objection_args = [utils.get_interpreter(), "-m", "objection.console.cli"]
             if environ.debug:
                 objection_args += ["--debug"]
             objection_args += ["-N", "-p", server.local_port]
@@ -74,9 +93,10 @@ class Command(AndroidCommand):
             if utils.is_empty(package):
                 target_app = server.get_frontmost_application()
                 if target_app is None:
-                    environ.logger.error("Unknown frontmost application")
-                    return 1
+                    raise CommandError("Unknown frontmost application")
                 package = target_app.identifier
+            environ.logger.info(f"Frida inject target application: {package}")
+
             objection_args += ["-g", package]
             objection_args += ["explore"]
 
@@ -93,12 +113,11 @@ class Command(AndroidCommand):
                 # 如果需要重定向到本地端口
                 address = args.redirect_address
                 port = args.redirect_port or 8080
-                info = device.get_package(package)
-                uid = info.user_id if info else None
+                uid = device.get_uid(package)
                 with device.redirect(address, port, uid):
-                    return utils.Popen(*objection_args).call()
+                    return utils.popen(*objection_args).call()
             else:
-                return utils.Popen(*objection_args).call()
+                return utils.popen(*objection_args).call()
 
 
 command = Command()
