@@ -21,47 +21,26 @@ limitations under the License.
 
 import re
 import sys
-from argparse import ArgumentParser
 from subprocess import PIPE
 from typing import Optional
 
-from linktools.cli.android import AndroidCommand
+from linktools import utils
+from linktools.cli import CommandParser, AndroidCommand, AndroidNamespace
 
 __version__ = '2.1.0'
 
 LOG_LEVELS = 'VDIWEF'
 LOG_LEVELS_MAP = dict([(LOG_LEVELS[i], i) for i in range(len(LOG_LEVELS))])
 
-widths = [
-    (126, 1), (159, 0), (687, 1), (710, 0), (711, 1),
-    (727, 0), (733, 1), (879, 0), (1154, 1), (1161, 0),
-    (4347, 1), (4447, 2), (7467, 1), (7521, 0), (8369, 1),
-    (8426, 0), (9000, 1), (9002, 2), (11021, 1), (12350, 2),
-    (12351, 1), (12438, 2), (12442, 0), (19893, 2), (19967, 1),
-    (55203, 2), (63743, 1), (64106, 2), (65039, 1), (65059, 0),
-    (65131, 2), (65279, 1), (65376, 2), (65500, 1), (65510, 2),
-    (120831, 1), (262141, 2), (1114109, 1),
-]
-
-
-def get_char_width(char):
-    global widths
-    o = ord(char)
-    if o == 0xe or o == 0xf:
-        return 0
-    for num, wid in widths:
-        if o <= num:
-            return wid
-    return 1
-
 
 class Command(AndroidCommand):
     """
     Filter logcat by package name
     """
 
-    def init_arguments(self, parser: ArgumentParser) -> None:
-        parser.add_argument('package', nargs='*', help='application package name(s)')
+    def init_arguments(self, parser: CommandParser) -> None:
+        parser.add_argument('-p', '--packages', dest='package', action='store', nargs='*', default=None,
+                           help='application package name(s)')
         parser.add_argument('-w', '--tag-width', metavar='N', dest='tag_width', type=int, default=23,
                             help='width of log tag')
         parser.add_argument('-l', '--min-level', dest='min_level', type=str, choices=LOG_LEVELS + LOG_LEVELS.lower(),
@@ -81,10 +60,9 @@ class Command(AndroidCommand):
         parser.add_argument('-a', '--all', dest='all', action='store_true', default=False,
                             help='print all log messages')
 
-    def run(self, args: [str]) -> Optional[int]:
-        args = self.parse_args(args)
-        device = args.parse_device()
-        package = args.package
+    def run(self, args: AndroidNamespace) -> Optional[int]:
+        device = args.device_selector.select()
+        package = args.package or []
         min_level = LOG_LEVELS_MAP[args.min_level.upper()]
 
         if args.current_app:
@@ -129,7 +107,7 @@ class Command(AndroidCommand):
             next = offset
             length = len(message)
             while next < length:
-                width = width - get_char_width(message[next])
+                width = width - utils.get_char_width(message[next])
                 if width < 0:
                     break
                 next = next + 1
@@ -202,8 +180,7 @@ class Command(AndroidCommand):
         PID_LINE = re.compile(r'^\w+\s+(\w+)\s+\w+\s+\w+\s+\w+\s+\w+\s+\w+\s+\w\s([\w|\.|\/]+)$')
         PID_START = re.compile(r'^.*: Start proc ([a-zA-Z0-9._:]+) for ([a-z]+ [^:]+): pid=(\d+) uid=(\d+) gids=(.*)$')
         PID_START_5_1 = re.compile(r'^.*: Start proc (\d+):([a-zA-Z0-9._:]+)/[a-z0-9]+ for (.*)$')
-        PID_START_DALVIK = re.compile(
-            r'^E/dalvikvm\(\s*(\d+)\): >>>>> ([a-zA-Z0-9._:]+) \[ userId:0 \| appId:(\d+) \]$')
+        PID_START_DALVIK = re.compile(r'^E/dalvikvm\(\s*(\d+)\): >>>>> ([a-zA-Z0-9._:]+) \[ userId:0 \| appId:(\d+) \]$')
         PID_KILL = re.compile(r'^Killing (\d+):([a-zA-Z0-9._:]+)/[^:]+: (.*)$')
         PID_LEAVE = re.compile(r'^No longer want ([a-zA-Z0-9._:]+) \(pid (\d+)\): .*$')
         PID_DEATH = re.compile(r'^Process ([a-zA-Z0-9._:]+) \(pid (\d+)\) has died.?$')
