@@ -22,26 +22,26 @@
       +-----------------------+  ;;  | |         |,"
          /_)______________(_/  //'   | +---------+
     ___________________________/___  `,
-   /  oooooooooooooooo  .o.  oooo /,   \,"-----------
-  / ==ooooooooooooooo==.o.  ooo= //   ,`\--{)B     ,"
+   /  oooooooooooooooo  .o.  oooo /,   `,"-----------
+  / ==ooooooooooooooo==.o.  ooo= //   ,``--{)B     ,"
  /_==__==========__==_ooo__ooo=_/'   /___________,"
 """
 
 import datetime
+import json
 import sys
-from argparse import ArgumentParser
 from typing import Optional
 
 from linktools import utils, environ
-from linktools.cli.android import AndroidCommand
+from linktools.cli import CommandParser, AndroidCommand, AndroidNamespace
 
 
 class Command(AndroidCommand):
     """
-    Fetch current running app's basic information
+    Fetch basic information about the currently running application
     """
 
-    def init_arguments(self, parser: ArgumentParser) -> None:
+    def init_arguments(self, parser: CommandParser) -> None:
         group = parser.add_mutually_exclusive_group()
         group.add_argument('-p', '--package', action='store_const', const=True, default=False,
                            help='show current package name')
@@ -56,43 +56,44 @@ class Command(AndroidCommand):
         group.add_argument('--screen', metavar='DEST', action='store', type=str, nargs='?', default=".",
                            help='capture screen and pull file')
 
-    def run(self, args: [str]) -> Optional[int]:
-        args = self.parse_args(args)
-        device = args.parse_device()
+    def run(self, args: AndroidNamespace) -> Optional[int]:
+        device = args.device_selector.select()
 
         if args.package:
-            environ.logger.info(device.get_current_package())
+            print(device.get_current_package())
         elif args.activity:
-            environ.logger.info(device.get_current_activity())
+            print(device.get_current_activity())
         elif args.path:
-            environ.logger.info(device.get_apk_path(device.get_current_package()))
+            print(device.get_apk_path(device.get_current_package()))
         elif args.kill:
             device.shell("am", "force-stop", device.get_current_package(), log_output=True)
         elif "--apk" in sys.argv:
-            package_name = device.get_current_package()
-            environ.logger.info("find current package: {}".format(package_name))
-            package = utils.get_item(device.get_packages(package_name, simple=True), 0)
-            if package is not None:
-                environ.logger.info("find current apk path: {}".format(package.source_dir))
-                path = device.get_storage_path("{}_{}.apk".format(package.name, package.version_name))
-                dest = args.apk if not utils.is_empty(args.apk) else "."
-                device.shell("mkdir", "-p", device.get_storage_path(), log_output=True)
-                device.shell("cp", package.source_dir, path, log_output=True)
-                device.pull(path, dest, log_output=True)
-                device.shell("rm", path)
+            name = device.get_current_package()
+            environ.logger.info(f"Find current package: {name}")
+            app = device.get_app(name)
+            environ.logger.info(f"Find current apk path: {app.source_dir}")
+            path = device.get_data_path("apk", "{}_{}.apk".format(app.name, app.version_name))
+            dest = args.apk if not utils.is_empty(args.apk) else "."
+            device.shell("mkdir", "-p", device.get_data_path("apk"), log_output=True)
+            device.shell("cp", app.source_dir, path, log_output=True)
+            device.exec("pull", path, dest, log_output=True)
+            device.shell("rm", path, ignore_errors=True)
         elif "--screen" in sys.argv:
             now = datetime.datetime.now()
-            path = device.get_storage_path("screenshot-" + now.strftime("%Y-%m-%d-%H-%M-%S") + ".png")
+            path = device.get_data_path("screenshot", "screenshot-" + now.strftime("%Y-%m-%d-%H-%M-%S") + ".png")
             dest = args.screen if not utils.is_empty(args.screen) else "."
-            device.shell("mkdir", "-p", device.get_storage_path(), log_output=True)
+            device.shell("mkdir", "-p", device.get_data_path("screenshot"), log_output=True)
             device.shell("screencap", "-p", path, log_output=True)
-            device.pull(path, dest, log_output=True)
-            device.shell("rm", path)
+            device.exec("pull", path, dest, log_output=True)
+            device.shell("rm", path, ignore_errors=True)
         else:
             package = device.get_current_package()
-            environ.logger.info("package:  ", package)
-            environ.logger.info("activity: ", device.get_current_activity())
-            environ.logger.info("path:     ", device.get_apk_path(package))
+            activity = device.get_current_activity()
+            environ.logger.info(json.dumps({
+                "package": package,
+                "activity": activity,
+                "path": device.get_apk_path(package),
+            }, ensure_ascii=False, indent=2))
 
         return
 
