@@ -22,31 +22,38 @@
       +-----------------------+  ;;  | |         |,"
          /_)______________(_/  //'   | +---------+
     ___________________________/___  `,
-   /  oooooooooooooooo  .o.  oooo /,   \,"-----------
-  / ==ooooooooooooooo==.o.  ooo= //   ,`\--{)B     ,"
+   /  oooooooooooooooo  .o.  oooo /,   `,"-----------
+  / ==ooooooooooooooo==.o.  ooo= //   ,``--{)B     ,"
  /_==__==========__==_ooo__ooo=_/'   /___________,"
 """
 import json
 import subprocess
-from argparse import ArgumentParser
-from typing import Optional
+from argparse import Namespace
+from typing import Optional, Type, List
 
-from linktools import environ
-from linktools.cli import BaseCommand
+from linktools.cli import BaseCommand, CommandMain, CommandParser
+from linktools.cli.argparse import KeyValueAction
+from linktools.types import DownloadError, ToolError
 
 
 class Command(BaseCommand):
     """
-    Tools downloaded from the web
+    Execute tools directly from remote URLs
     """
 
-    _TOOL_NAMES = sorted([tool.name for tool in iter(environ.tools)])
+    @property
+    def main(self) -> CommandMain:
+        return CommandMain(self, show_log_level=False, show_log_time=False)
 
-    def init_arguments(self, parser: ArgumentParser) -> None:
-        group = parser.add_mutually_exclusive_group()
-        parser.add_argument("--set", metavar=("KEY", "VALUE"),
-                            action="append", nargs=2, dest="configs", default=[],
+    @property
+    def known_errors(self) -> List[Type[BaseException]]:
+        return super().known_errors + [ToolError, DownloadError]
+
+    def init_arguments(self, parser: CommandParser) -> None:
+        parser.add_argument("--set", action=KeyValueAction, nargs=1, dest="configs",
                             help="set the config of tool")
+
+        group = parser.add_mutually_exclusive_group()
         group.add_argument("-c", "--config", action="store_true", default=False,
                            help="show the config of tool")
         group.add_argument("--download", action="store_true", default=False,
@@ -55,31 +62,32 @@ class Command(BaseCommand):
                            help="clear tool files")
         group.add_argument("-d", "--daemon", action="store_true", default=False,
                            help="execute tools as a daemon")
-        parser.add_argument("tool", nargs="...", choices=self._TOOL_NAMES)
 
-    def run(self, args: [str]) -> Optional[int]:
-        args = self.parse_args(args)
-        if len(args.tool) == 0 or args.tool[0] not in self._TOOL_NAMES:
-            self.print_help()
-            return -1
+        tool_names = sorted([tool.name for tool in self.environ.tools.values()])
+        subparsers = parser.add_subparsers(metavar="TOOL", help=f"{{{','.join(tool_names)}}}")
+        subparsers.required = True
+        for tool_name in tool_names:
+            tool_parser = subparsers.add_parser(tool_name, prefix_chars=chr(0), add_help=False)
+            tool_parser.add_argument("tool_args", metavar="args", nargs="...")
+            tool_parser.set_defaults(tool_name=tool_name)
 
-        tool_name = args.tool[0]
-        tool_args = args.tool[1:]
-        tool = environ.get_tool(tool_name, **{k: v for k, v in args.configs})
+    def run(self, args: Namespace) -> Optional[int]:
+
+        tool_name, tool_args = args.tool_name, args.tool_args
+        tool = self.environ.get_tool(tool_name, **(args.configs or {}))
 
         if args.config:
-            environ.logger.info(json.dumps(tool.config, indent=2, ensure_ascii=False))
+            self.logger.info(json.dumps(tool.config, indent=2, ensure_ascii=False))
             return 0
 
         elif args.download:
-            if not tool.exists:
-                tool.prepare()
-            environ.logger.info(f"Download tool files success: {tool.absolute_path}")
+            tool.prepare()
+            self.logger.info(f"Download tool files success: {tool.absolute_path}")
             return 0
 
         elif args.clear:
             tool.clear()
-            environ.logger.info(f"Clear tool files success")
+            self.logger.info(f"Clear tool files success")
             return 0
 
         elif args.daemon:
@@ -87,9 +95,12 @@ class Command(BaseCommand):
                 *tool_args,
                 stdin=subprocess.DEVNULL,
                 stdout=subprocess.DEVNULL,
-                stderr=subprocess.DEVNULL,
+                stderr=None,
             )
-            return process.call_as_daemon()
+            try:
+                return process.wait(timeout=1)
+            except subprocess.TimeoutExpired:
+                return None
 
         else:
             process = tool.popen(*tool_args)
