@@ -22,38 +22,41 @@
       +-----------------------+  ;;  | |         |,"
          /_)______________(_/  //'   | +---------+
     ___________________________/___  `,
-   /  oooooooooooooooo  .o.  oooo /,   \,"-----------
-  / ==ooooooooooooooo==.o.  ooo= //   ,`\--{)B     ,"
+   /  oooooooooooooooo  .o.  oooo /,   `,"-----------
+  / ==ooooooooooooooo==.o.  ooo= //   ,``--{)B     ,"
  /_==__==========__==_ooo__ooo=_/'   /___________,"
 """
 import re
-from argparse import ArgumentParser
-from typing import Optional
+from typing import Optional, Type, List
 
-from linktools import utils, environ
-from linktools.cli.ios import IOSCommand
-from linktools.frida import FridaApplication, FridaShareScript, FridaScriptFile, FridaEvalCode
-from linktools.frida.ios import IOSFridaServer
+from linktools import utils
+from linktools.cli import CommandError, CommandMain, CommandParser, IOSCommand, IOSNamespace
+from linktools.cli.argparse import KeyValueAction, range_type, BooleanOptionalAction
+from linktools.frida import FridaApplication, FridaShareScript, FridaScriptFile, FridaEvalCode, FridaIOSServer
+from linktools.types import DownloadError
 
 
 class Command(IOSCommand):
     """
-    Easy to use frida (require iOS device jailbreak)
+    Use Frida for dynamic analysis on jailbroken iOS devices
     """
 
-    def main(self, *args, **kwargs) -> None:
-        environ.set_config("SHOW_LOG_TIME", True)
-        environ.set_config("SHOW_LOG_LEVEL", True)
-        return super().main(*args, **kwargs)
+    @property
+    def main(self) -> CommandMain:
+        return CommandMain(self, show_log_level=True, show_log_time=True)
 
-    def init_arguments(self, parser: ArgumentParser) -> None:
+    @property
+    def known_errors(self) -> List[Type[BaseException]]:
+        return super().known_errors + [DownloadError]
+
+    def init_arguments(self, parser: CommandParser) -> None:
         parser.add_argument("-b", "--bundle-id", action="store", default=None,
                             help="target bundle id (default: frontmost application)")
         parser.add_argument("--spawn", action="store_true", default=False,
                             help="inject after spawn (default: false)")
 
-        parser.add_argument("-P", "--parameters", metavar=("KEY", "VALUE"),
-                            action="append", nargs=2, dest="user_parameters", default=[],
+        parser.add_argument("-P", "--parameters",
+                            action=KeyValueAction, nargs="+", dest="user_parameters",
                             help="user script parameters")
 
         parser.add_argument("-l", "--load", metavar="SCRIPT",
@@ -70,45 +73,65 @@ class Command(IOSCommand):
         parser.add_argument("-a", "--auto-start", action="store_true", default=False,
                             help="automatically start when all processes exits")
 
-    def run(self, args: [str]) -> Optional[int]:
-        args = self.parse_args(args)
-        device = args.parse_device()
-        bundle_id = args.bundle_id
+        parser.add_argument("--local-port", metavar="PORT", action="store", dest="local_port",
+                            type=range_type(1, 65536), default=None,
+                            help="local frida port (default: unused port)")
+        parser.add_argument("--remote-port", metavar="PORT", action="store", dest="remote_port",
+                            type=range_type(1, 65536), default=27042,
+                            help="remote frida port (default: 27042)")
 
-        user_parameters = {p[0]: p[1] for p in args.user_parameters}
+        parser.add_argument("--spawn-gating", action=BooleanOptionalAction, default=True,
+                            help="enable spawn gating (default: true)")
+        parser.add_argument("--child-gating", action=BooleanOptionalAction, default=False,
+                            help="enable child gating (default: false)")
+
+    def run(self, args: IOSNamespace) -> Optional[int]:
+
+        logger = self.logger
+        user_parameters = args.user_parameters
         user_scripts = args.user_scripts
 
+        device = args.device_selector.select()
+        bundle_id = args.bundle_id
+
         class Application(FridaApplication):
 
             def on_session_detached(self, session, reason, crash) -> None:
-                environ.logger.info(f"{session} detached, reason={reason}")
+                logger.info(f"{session} detached, reason={reason}")
                 if reason in ("connection-terminated", "device-lost"):
-                    self.stop()
+                    self.signal_stop()
                 elif len(self.sessions) == 0:
                     if args.auto_start:
-                        app.load_script(app.device.spawn(bundle_id), resume=True)
+                        app.spawn(bundle_id)
 
-        with IOSFridaServer(device=device, local_port=utils.pick_unused_port()) as server:
+        server = FridaIOSServer(
+            device=device,
+            local_port=args.local_port or utils.get_free_port(),
+            remote_port=args.remote_port,
+        )
 
+        with server:
             # 如果没有填包名，则找到顶层应用
             if utils.is_empty(bundle_id):
                 target_app = server.get_frontmost_application()
                 if target_app is None:
-                    environ.logger.error("Unknown frontmost application")
-                    return
+                    raise CommandError("Unknown frontmost application")
                 bundle_id = target_app.identifier
 
+            logger.info(f"Frida inject target application: {bundle_id}")
+
             app = Application(
                 server,
                 target_identifiers=rf"^{re.escape(bundle_id)}$",
                 user_parameters=user_parameters,
                 user_scripts=user_scripts,
-                enable_spawn_gating=True
+                enable_spawn_gating=args.spawn_gating,
+                enable_child_gating=args.child_gating
             )
 
             if args.spawn:
                 # 打开进程后注入
-                app.load_script(app.device.spawn(bundle_id), resume=True)
+                app.spawn(bundle_id)
 
             elif app.inject_all():
                 # 注入所有进程进程
@@ -116,9 +139,9 @@ class Command(IOSCommand):
 
             elif args.auto_start:
                 # 进程不存在，打开进程后注入
-                app.load_script(app.device.spawn(bundle_id), resume=True)
+                app.spawn(bundle_id)
 
-            app.run()
+            return app.run()
 
 
 command = Command()
