@@ -22,24 +22,29 @@
       +-----------------------+  ;;  | |         |,"
          /_)______________(_/  //'   | +---------+
     ___________________________/___  `,
-   /  oooooooooooooooo  .o.  oooo /,   \,"-----------
-  / ==ooooooooooooooo==.o.  ooo= //   ,`\--{)B     ,"
+   /  oooooooooooooooo  .o.  oooo /,   `,"-----------
+  / ==ooooooooooooooo==.o.  ooo= //   ,``--{)B     ,"
  /_==__==========__==_ooo__ooo=_/'   /___________,"
 """
-from argparse import ArgumentParser
-from typing import Optional
+from typing import Optional, List, Type
 
 from linktools import utils, environ
-from linktools.cli.ios import IOSCommand
-from linktools.frida.ios import IOSFridaServer
+from linktools.cli import CommandParser, CommandError, IOSCommand, IOSNamespace
+from linktools.cli.argparse import range_type
+from linktools.frida import FridaIOSServer
+from linktools.types import DownloadError
 
 
 class Command(IOSCommand):
     """
-    Easy to use objection (require iOS device jailbreak)
+    Simplify security testing with Objection on jailbroken devices
     """
 
-    def init_arguments(self, parser: ArgumentParser) -> None:
+    @property
+    def known_errors(self) -> List[Type[BaseException]]:
+        return super().known_errors + [DownloadError]
+
+    def init_arguments(self, parser: CommandParser) -> None:
         parser.add_argument("-b", "--bundle-id", action="store", default=None,
                             help="target bundle id (default: frontmost application)")
         parser.add_argument("-s", "--startup-command", action="append", default=[],
@@ -52,13 +57,24 @@ class Command(IOSCommand):
         parser.add_argument("-P", "--plugin-folder", action="store", default=environ.get_asset_path("objection"),
                             help="The folder to load plugins from.")
 
-    def run(self, args: [str]) -> Optional[int]:
-        args = self.parse_args(args)
-        device = args.parse_device()
+        parser.add_argument("--local-port", metavar="PORT", action="store", dest="local_port",
+                            type=range_type(1, 65536), default=None,
+                            help="local frida port (default: unused port)")
+        parser.add_argument("--remote-port", metavar="PORT", action="store", dest="remote_port",
+                            type=range_type(1, 65536), default=27042,
+                            help="remote frida port (default: 27042)")
 
-        with IOSFridaServer(device=device, local_port=utils.pick_unused_port()) as server:
+    def run(self, args: IOSNamespace) -> Optional[int]:
+        device = args.device_selector.select()
 
-            objection_args = ["objection"]
+        server = FridaIOSServer(
+            device=device,
+            local_port=args.local_port or utils.get_free_port(),
+            remote_port=args.remote_port,
+        )
+
+        with server:
+            objection_args = [utils.get_interpreter(), "-m", "objection.console.cli"]
             if environ.debug:
                 objection_args += ["--debug"]
             objection_args += ["-N", "-p", server.local_port]
@@ -67,9 +83,10 @@ class Command(IOSCommand):
             if utils.is_empty(bundle_id):
                 target_app = server.get_frontmost_application()
                 if target_app is None:
-                    environ.logger.error("Unknown frontmost application")
-                    return 1
+                    raise CommandError("Unknown frontmost application")
                 bundle_id = target_app.identifier
+            environ.logger.info(f"Frida inject target application: {bundle_id}")
+
             objection_args += ["-g", bundle_id]
             objection_args += ["explore"]
 
@@ -82,7 +99,7 @@ class Command(IOSCommand):
             if args.plugin_folder:
                 objection_args += ["--plugin-folder", args.plugin_folder]
 
-            return utils.Popen(*objection_args).call()
+            return utils.popen(*objection_args).call()
 
 
 command = Command()
