@@ -2,18 +2,16 @@
 # -*- coding:utf-8 -*-
 
 import os
-from argparse import ArgumentParser
-from typing import Optional, Tuple, Type
+from typing import Optional, Type, List
 
 import paramiko
 from paramiko.ssh_exception import SSHException
 
 from linktools import utils
-from linktools.cli.ios import IOSCommand
-from linktools.ios import Device
+from linktools.cli import CommandParser, IOSCommand, IOSNamespace
 from linktools.ssh import SSHClient
 
-_REMOTE_PATH_PREFIX = "@"
+_REMOTE_PATH_PREFIX = ":"
 
 
 class SCPFile(os.PathLike):
@@ -35,14 +33,14 @@ class SCPFile(os.PathLike):
 
 class Command(IOSCommand):
     """
-    OpenSSH secure file copy (require iOS device jailbreak)
+    Securely copy files to/from a jailbroken iOS device using OpenSSH
     """
 
     @property
-    def known_errors(self) -> Tuple[Type[BaseException]]:
-        return super().known_errors + tuple([NotImplementedError, FileNotFoundError, SSHException])
+    def known_errors(self) -> List[Type[BaseException]]:
+        return super().known_errors + [NotImplementedError, FileNotFoundError, SSHException]
 
-    def init_arguments(self, parser: ArgumentParser) -> None:
+    def init_arguments(self, parser: CommandParser) -> None:
         parser.add_argument("-u", "--username", action="store", default="root",
                             help="iOS ssh username (default: root)")
         parser.add_argument("-p", "--port", action="store", type=int, default=22,
@@ -55,11 +53,10 @@ class Command(IOSCommand):
         parser.add_argument("target", action="store", type=SCPFile, default=None,
                             help=f"target file path, remote path needs to be prefixed with \"{_REMOTE_PATH_PREFIX}\"")
 
-    def run(self, args: [str]) -> Optional[int]:
-        args = self.parse_args(args)
-        device: Device = args.parse_device()
+    def run(self, args: IOSNamespace) -> Optional[int]:
+        device = args.device_selector.select()
 
-        local_port = utils.pick_unused_port()
+        local_port = utils.get_free_port()
         with device.forward(local_port, args.port):
             with SSHClient() as client:
                 client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
