@@ -22,44 +22,46 @@
       +-----------------------+  ;;  | |         |,"
          /_)______________(_/  //'   | +---------+
     ___________________________/___  `,
-   /  oooooooooooooooo  .o.  oooo /,   \,"-----------
-  / ==ooooooooooooooo==.o.  ooo= //   ,`\--{)B     ,"
+   /  oooooooooooooooo  .o.  oooo /,   `,"-----------
+  / ==ooooooooooooooo==.o.  ooo= //   ,``--{)B     ,"
  /_==__==========__==_ooo__ooo=_/'   /___________,"
 """
-from argparse import ArgumentParser
-from typing import Optional, Tuple, Type
+from typing import Optional, Type, List
 
 import paramiko
 from paramiko.ssh_exception import SSHException
 
 from linktools import utils
-from linktools.cli.ios import IOSCommand
+from linktools.cli import CommandParser, CommandMain, IOSCommand, IOSNamespace
 from linktools.ssh import SSHClient
 
 
 class Command(IOSCommand):
     """
-    OpenSSH remote login client (require iOS device jailbreak)
+    Remotely login to jailbroken iOS devices using the OpenSSH client
     """
 
     @property
-    def known_errors(self) -> Tuple[Type[BaseException]]:
-        return super().known_errors + tuple([SSHException])
+    def main(self) -> CommandMain:
+        return CommandMain(self, show_log_level=False, show_log_time=False)
 
-    def init_arguments(self, parser: ArgumentParser) -> None:
+    @property
+    def known_errors(self) -> List[Type[BaseException]]:
+        return super().known_errors + [SSHException]
+
+    def init_arguments(self, parser: CommandParser) -> None:
         parser.add_argument("-u", "--username", action="store", default="root",
                             help="iOS ssh username (default: root)")
         parser.add_argument("-p", "--port", action="store", type=int, default=22,
                             help="iOS ssh port (default: 22)")
         parser.add_argument("--password", action="store",
                             help="iOS ssh password")
-        parser.add_argument('ssh_args', nargs='...', help="ssh args")
+        parser.add_argument("ssh_args", nargs="...", help="ssh args")
 
-    def run(self, args: [str]) -> Optional[int]:
-        args = self.parse_args(args)
-        device = args.parse_device()
+    def run(self, args: IOSNamespace) -> Optional[int]:
+        device = args.device_selector.select()
 
-        local_port = utils.pick_unused_port()
+        local_port = utils.get_free_port()
         with device.forward(local_port, args.port):
             with SSHClient() as client:
                 client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
