@@ -22,11 +22,11 @@
       +-----------------------+  ;;  | |         |,"
          /_)______________(_/  //'   | +---------+
     ___________________________/___  `,
-   /  oooooooooooooooo  .o.  oooo /,   \,"-----------
-  / ==ooooooooooooooo==.o.  ooo= //   ,`\--{)B     ,"
+   /  oooooooooooooooo  .o.  oooo /,   `,"-----------
+  / ==ooooooooooooooo==.o.  ooo= //   ,``--{)B     ,"
  /_==__==========__==_ooo__ooo=_/'   /___________,"
 """
 
 from .app import FridaApplication
 from .script import FridaScriptFile, FridaShareScript, FridaEvalCode
-from .server import FridaServer
+from .server import FridaServer, FridaAndroidServer, FridaIOSServer
