@@ -12,18 +12,20 @@ import logging
 import os
 import re
 import threading
-from typing import Optional, Union, Dict, Collection, Callable, Any
+from typing import TYPE_CHECKING, Optional, Union, Dict, Collection, Callable, Any, List, Iterable
 
-import _frida
 import frida
 from frida.core import Session, Script
 
-from ._utils import Counter
 from .script import FridaUserScript, FridaEvalCode, FridaScriptFile
 from .server import FridaServer
-from .. import utils, environ
+from .. import utils, environ, metadata
+from ..decorator import timeoutable, cached_property
 from ..reactor import Reactor
-from ..version import __release__
+from ..types import TimeoutType, Stoppable
+
+if TYPE_CHECKING:
+    import _frida
 
 _logger = environ.get_logger("frida.app")
 
@@ -46,7 +48,7 @@ class FridaSession(utils.get_derived_type(Session)):  # proxy for frida.core.Ses
     def __init__(self, session: Session, name: str = None):
         super().__init__(session)
         self._name: str = name or ""
-        self._scripts: [FridaScript] = []
+        self._scripts: List[FridaScript] = []
 
     @property
     def pid(self) -> int:
@@ -57,7 +59,7 @@ class FridaSession(utils.get_derived_type(Session)):  # proxy for frida.core.Ses
         return self._name
 
     @property
-    def scripts(self) -> ["FridaScript"]:
+    def scripts(self) -> List["FridaScript"]:
         return self._scripts
 
     @property
@@ -101,7 +103,7 @@ class FridaScript(utils.get_derived_type(Script)):  # proxy for frida.core.Scrip
         return self._session
 
     @property
-    def exports_sync(self):
+    def exports_sync(self) -> "frida.core.ScriptExportsSync":
         if hasattr(self.__super__, "exports_sync"):
             return self.__super__.exports_sync
         return self.__super__.exports
@@ -190,7 +192,7 @@ class FridaScriptHandler(metaclass=abc.ABCMeta):
             if payload and isinstance(payload, dict):
 
                 # 单独解析Emitter发出来的消息
-                events = payload.pop("$events", None)
+                events: Optional[List[Any]] = payload.pop("$events", None)
                 if events:
                     for event in events:
                         # 如果消息类型是log，那就直接调on_log
@@ -199,10 +201,20 @@ class FridaScriptHandler(metaclass=abc.ABCMeta):
                             level = log.get("level") or self.LogLevel.DEBUG
                             message = log.get("message")
                             self.on_script_log(script, level, message, data)
+
                         # 如果只是普通消息，则调用on_event
                         msg = event.pop("msg", None)
                         if msg is not None:
-                            self.on_script_event(script, msg, data)
+                            try:
+                                self.on_script_event(script, msg, data)
+                            except Exception as e:
+                                _logger.error(f"on_script_event error", exc_info=e)
+
+                        # 如果是error消息，则调用on_log展示stack
+                        error = event.pop("error", None)
+                        if error is not None:
+                            stack = utils.get_item(error, "stack")
+                            self.on_script_log(script, self.LogLevel.ERROR, stack if stack else error, data)
 
                     return
 
@@ -239,15 +251,15 @@ class FridaScriptHandler(metaclass=abc.ABCMeta):
         if not utils.is_empty(message):
             log_fn(message)
 
-    def on_script_event(self, script: FridaScript, message: Any, data: Any):
+    def on_script_event(self, script: FridaScript, event: Any, data: Any):
         """
         脚本发送事件回调
         :param script: frida的脚本
-        :param message: 事件消息
+        :param event: 事件消息
         :param data: 事件带回来的数据
         """
         message = f"{script} event: {os.linesep}" \
-                  f"{json.dumps(message, indent=2, ensure_ascii=False)}"
+                  f"{json.dumps(event, indent=2, ensure_ascii=False)}"
         self.on_script_log(script, self.LogLevel.INFO, message, None)
 
     def on_script_send(self, script: FridaScript, payload: Any, data: Any):
@@ -279,14 +291,54 @@ class FridaFileHandler(metaclass=abc.ABCMeta):
         _logger.debug(f"{file} changed")
 
 
+class FridaEventCounter:
+
+    def __init__(self):
+        self._map = {}
+        self._lock = threading.RLock()
+
+    def increase(self, group: "Group" = None) -> int:
+        with self._lock:
+            keys = group.values if group else tuple()
+            if keys not in self._map:
+                self._map[keys] = 0
+            result = self._map[keys] = self._map[keys] + 1
+            return result
+
+    class Group:
+
+        def __init__(self, accept_empty: bool = False):
+            self._accept_empty = accept_empty
+            self._names = []
+            self._values = []
+
+        def add(self, **kwargs):
+            for k, v in kwargs.items():
+                if self._accept_empty or v is not None:
+                    self._names.append(k)
+                    self._values.append(v)
+            return self
+
+        @property
+        def names(self):
+            return tuple(self._names)
+
+        @property
+        def values(self):
+            return tuple(self._values)
+
+        def __repr__(self):
+            return f"Group({', '.join(self._names)})"
+
+
 class FridaManager:
 
     def __init__(self, reactor: FridaReactor):
         self._reactor = reactor
-        self._cancel_handlers = {}
+        self._cancel_handlers: "Dict[str, Callable[[], Any]]" = {}
 
         self._lock = threading.RLock()
-        self._sessions: Dict[int, FridaSession] = {}
+        self._sessions: "Dict[int, FridaSession]" = {}
 
     @property
     def sessions(self) -> Dict[int, FridaSession]:
@@ -314,7 +366,7 @@ class FridaManager:
             self._sessions[session.pid] = session
 
     def add_device_handler(self, device: frida.core.Device, handler: FridaDeviceHandler):
-        self._cancel(device)
+        self._call_cancel_handler(device)
 
         cb_spawn_added = lambda spawn: threading.Thread(target=handler.on_spawn_added, args=(spawn,)).start()
         cb_spawn_removed = lambda spawn: self._reactor.schedule(lambda: handler.on_spawn_removed(spawn))
@@ -333,25 +385,25 @@ class FridaManager:
         device.on("lost", cb_lost)
 
         def cancel():
-            utils.ignore_error(device.off, args=("spawn-added", cb_spawn_added))
-            utils.ignore_error(device.off, args=("spawn-removed", cb_spawn_removed))
-            utils.ignore_error(device.off, args=("child-added", cb_child_added))
-            utils.ignore_error(device.off, args=("child-removed", cb_child_removed))
-            utils.ignore_error(device.off, args=("output", cb_output))
-            # utils.ignore_error(device.off, args=("process-crashed", cb_process_crashed))
-            # utils.ignore_error(device.off, args=("uninjected", cb_uninjected))
-            utils.ignore_error(device.off, args=("lost", cb_lost))
+            utils.ignore_errors(device.off, args=("spawn-added", cb_spawn_added))
+            utils.ignore_errors(device.off, args=("spawn-removed", cb_spawn_removed))
+            utils.ignore_errors(device.off, args=("child-added", cb_child_added))
+            utils.ignore_errors(device.off, args=("child-removed", cb_child_removed))
+            utils.ignore_errors(device.off, args=("output", cb_output))
+            # utils.ignore_errors(device.off, args=("process-crashed", cb_process_crashed))
+            # utils.ignore_errors(device.off, args=("uninjected", cb_uninjected))
+            utils.ignore_errors(device.off, args=("lost", cb_lost))
 
-        self._register_cancel(device, cancel)
+        self._register_cancel_handler(device, cancel)
 
     def remove_device_handler(self, device: frida.core.Device):
-        self._cancel(device)
+        self._call_cancel_handler(device)
 
     def add_session_handler(self, session: FridaSession, handler: FridaSessionHandler):
-        self._cancel(session)
+        self._call_cancel_handler(session)
 
         def on_detached(reason, crash):
-            self._reactor.schedule(lambda: self._cancel(session))
+            self._reactor.schedule(lambda: self._call_cancel_handler(session))
             with self._lock:
                 self._sessions.pop(session.pid, None)
             self._reactor.schedule(lambda: handler.on_session_detached(session, reason, crash))
@@ -359,44 +411,44 @@ class FridaManager:
         session.on("detached", on_detached)
 
         def cancel():
-            utils.ignore_error(session.off, args=("detached", on_detached))
+            utils.ignore_errors(session.off, args=("detached", on_detached))
 
-        self._register_cancel(session, cancel)
+        self._register_cancel_handler(session, cancel)
 
     def remove_session_handler(self, session: FridaSession):
-        self._cancel(session)
+        self._call_cancel_handler(session)
 
     def add_script_handler(self, script: FridaScript, handler: FridaScriptHandler):
-        self._cancel(script)
+        self._call_cancel_handler(script)
 
         def on_message(msg, data):
             return handler.on_script_message(script, msg, data)
 
         def on_destroyed():
-            self._reactor.schedule(lambda: self._cancel(script))
+            self._reactor.schedule(lambda: self._call_cancel_handler(script))
             return handler.on_script_destroyed(script)
 
         script.on("message", on_message)
         script.on("destroyed", on_destroyed)
 
         def cancel():
-            utils.ignore_error(script.off, args=("message", on_message))
-            utils.ignore_error(script.off, args=("destroyed", on_destroyed))
+            utils.ignore_errors(script.off, args=("message", on_message))
+            utils.ignore_errors(script.off, args=("destroyed", on_destroyed))
 
-        self._register_cancel(script, cancel)
+        self._register_cancel_handler(script, cancel)
 
     def remove_script_handler(self, script: FridaScript):
-        self._cancel(script)
+        self._call_cancel_handler(script)
 
-    def add_file_handler(self, files: [FridaScriptFile], handler: FridaFileHandler):
-        self._cancel(files)
+    def add_file_handler(self, files: Iterable[FridaScriptFile], handler: FridaFileHandler):
+        self._call_cancel_handler(files)
 
         last_change_id = 0
         monitors: Dict[str, frida.FileMonitor] = {}
 
         def make_monitor(file):
             _logger.debug(f"Monitor file: {file.path}")
-            monitor = frida.FileMonitor(file.path)
+            monitor = frida.FileMonitor(str(file.path))
             monitor.on("change", lambda changed_file, other_file, event_type: on_change_handler(event_type, file))
             monitor.enable()
             return monitor
@@ -423,15 +475,15 @@ class FridaManager:
             for monitor in monitors.values():
                 monitor.disable()
 
-        self._register_cancel(files, cancel)
+        self._register_cancel_handler(files, cancel)
 
-    def remove_file_handler(self, files: [FridaScriptFile]):
-        self._cancel(files)
+    def remove_file_handler(self, files: Iterable[FridaScriptFile]):
+        self._call_cancel_handler(files)
 
-    def _register_cancel(self, key: Any, handler: Callable[[], Any]):
+    def _register_cancel_handler(self, key: Any, handler: Callable[[], Any]):
         self._cancel_handlers[self._make_key(key)] = handler
 
-    def _cancel(self, key: Any):
+    def _call_cancel_handler(self, key: Any):
         handler = self._cancel_handlers.pop(self._make_key(key), None)
         if handler:
             handler()
@@ -443,7 +495,7 @@ class FridaManager:
         return key
 
 
-class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandler, FridaFileHandler):
+class FridaApplication(Stoppable, FridaDeviceHandler, FridaSessionHandler, FridaScriptHandler, FridaFileHandler):
     """
     ----------------------------------------------------------------------
 
@@ -494,14 +546,15 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
 
         # 初始化运行环境
         self._last_error = None
-        self._stop_request = utils.InterruptableEvent()
-        self._finished = utils.InterruptableEvent()
+        self._stop_request = threading.Event()
         self._reactor = FridaReactor(on_stop=self._on_stop, on_error=self._on_error)
         self._manager = FridaManager(self._reactor)
 
         # 初始化内置脚本
-        script_name = "frida.js" if not __release__ or environ.debug else "frida.min.js"
-        self._internal_script = FridaScriptFile(environ.get_asset_path(script_name))
+        script_path = environ.get_asset_path("frida.min.js")
+        if metadata.__develop__ or environ.debug or not os.path.exists(script_path):
+            script_path = environ.get_asset_path("frida.js")
+        self._internal_script = FridaScriptFile(script_path)
 
         # 初始化需要注入进程的匹配规则
         if isinstance(target_identifiers, str):
@@ -509,7 +562,7 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
         elif isinstance(target_identifiers, Collection):
             self._target_identifiers = [re.compile(i) for i in target_identifiers]
         else:
-            self._target_identifiers: [re.Pattern] = []
+            self._target_identifiers: Iterable[re.Pattern] = []
 
         # 初始化用户传递的参数
         self._user_parameters = user_parameters or {}
@@ -520,7 +573,7 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
         elif isinstance(user_scripts, Collection):
             self._user_scripts = user_scripts
         else:
-            self._user_scripts: [FridaUserScript] = []
+            self._user_scripts: Iterable[FridaUserScript] = []
 
         # 初始化所有需要监听的脚本文件
         self._user_files = []
@@ -535,8 +588,6 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
         self._enable_child_gating = enable_child_gating
         self._eternalize = eternalize
 
-        self._event_counter = Counter()
-
     @property
     def device(self) -> frida.core.Device:
         return self._device
@@ -547,93 +598,135 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
         for user_script in self._user_scripts:
             user_script.load()
 
-        self._finished.clear()
+        self._stop_request.clear()
+
         self._manager.add_file_handler(self._user_files, self)
         self._manager.add_device_handler(self.device, self)
 
         if self._enable_spawn_gating:
-            self.device.enable_spawn_gating()
+            try:
+                self.device.enable_spawn_gating()
+            except frida.NotSupportedError:
+                _logger.warning(f"Enable child gating is not supported, ignore")
+        else:
+            try:
+                self.device.disable_spawn_gating()
+            except frida.NotSupportedError:
+                pass
 
     def _deinit(self):
         _logger.debug(f"FridaApplication deinit")
 
         self._manager.remove_device_handler(self.device)
         self._manager.remove_file_handler(self._user_files)
-        self._finished.set()
 
     @property
     def is_running(self) -> bool:
         return self._reactor.is_running()
 
-    @utils.timeoutable
-    def run(self, timeout: utils.Timeout = None):
+    def start(self):
         assert not self.is_running
         try:
             self._init()
-            with self._reactor:
-                self._stop_request.wait(timeout)
-        finally:
-            self._deinit()
+            self._reactor.start()
+        except:
+            self.stop()
+            raise
 
-    @utils.timeoutable
-    def wait(self, timeout: utils.Timeout = None) -> bool:
-        return self._finished.wait(timeout)
+    @timeoutable
+    def run(self, timeout: TimeoutType = None):
+        assert not self.is_running
+        try:
+            self._init()
+            self._reactor.start()
+            utils.wait_event(self._stop_request, timeout)
+        finally:
+            self.stop()
+
+    @timeoutable
+    def wait(self, timeout: TimeoutType = None) -> bool:
+        return utils.wait_event(self._stop_request, timeout)
 
     def stop(self):
+        self._reactor.signal_stop()
+        if not self._reactor.wait(5):
+            _logger.warning("Worker did not finish normally")
+        self._deinit()
+
+    def signal_stop(self):
         self._stop_request.set()
+        self._reactor.signal_stop()
 
     def __enter__(self):
         assert not self.is_running
-        try:
-            self._init()
-            self._reactor.run()
-            return self
-        except Exception as e:
-            self._deinit()
-            raise e
+        self.start()
+        return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        self._reactor.stop()
-        self._reactor.wait()
-        self._deinit()
+        self.stop()
+
+    @cached_property(lock=True)
+    def counter(self) -> FridaEventCounter:
+        return FridaEventCounter()
 
     def schedule(self, fn: Callable[[], any], delay: float = None):
         self._reactor.schedule(fn, delay)
 
-    def load_script(self, pid, resume=False):
+    def load_script(self, process_id: int, process_name: str = None, resume: bool = True):
         """
         加载脚本，注入到指定进程
-        :param pid: 进程id
+        :param process_id: 进程id
+        :param process_name: 进程名称
         :param resume: 注入后是否需要resume进程
         """
-        self._reactor.schedule(lambda: self._load_script(pid, resume))
+        self._reactor.schedule(lambda: self._load_script(process_id, process_name, resume))
 
-    def inject_all(self) -> [int]:
+    def spawn(self, program: str, resume: bool = True, **kwargs) -> int:
+        """
+        启动进程，若进程匹配target_identifiers则注入脚本
+        """
+        pid = self.device.spawn(program, **kwargs)
+        for identifier in self._target_identifiers:
+            if identifier.search(program):
+                self.load_script(pid, program, resume=resume)
+                break
+        else:
+            if resume:
+                self._reactor.schedule(lambda: utils.ignore_errors(self.device.resume, args=(pid,)))
+        return pid
+
+    def inject_all(self, resume: bool = True) -> Iterable[int]:
         """
         根据target_identifiers注入所有匹配的进程
         :return: 注入的进程pid
         """
 
-        target_pids = set()
-
+        # 匹配所有app的进程id
+        app_pids = set()
+        app_list = self.device.enumerate_applications()
         for identifier in self._target_identifiers:
+            for app in app_list:
+                if app.pid > 0:
+                    if identifier.search(app.identifier):
+                        app_pids.add(app.pid)
 
-            # 匹配所有app
-            for target_app in self.device.enumerate_applications():
-                if target_app.pid > 0 and identifier.search(target_app.identifier):
-                    target_pids.add(target_app.pid)
+        # 匹配所有进程
+        processes = set()
+        process_list = self.device.enumerate_processes()
+        for identifier in self._target_identifiers:
+            for process in process_list:
+                if process.pid > 0:
+                    if process.pid in app_pids or identifier.search(process.name):
+                        processes.add(process)
 
-            # 匹配所有进程
-            for target_process in self.device.enumerate_processes():
-                if target_process.pid > 0 and identifier.search(target_process.name):
-                    target_pids.add(target_process.pid)
-
-        if len(target_pids) > 0:
+        pids = list()
+        if len(processes) > 0:
             # 进程存在，直接注入
-            for pid in target_pids:
-                self.load_script(pid)
+            for process in processes:
+                self.load_script(process.pid, process.name, resume=resume)
+                pids.append(process.pid)
 
-        return target_pids
+        return pids
 
     @property
     def sessions(self) -> Dict[int, FridaSession]:
@@ -677,49 +770,62 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
 
         return [o.as_dict() for o in script_files]
 
-    def _load_script(self, pid: int, resume: bool = False):
-        _logger.debug(f"Attempt to load script: pid={pid}, resume={resume}")
+    def _load_script(self, process_id: int, process_name: str, resume: bool):
+        _logger.debug(f"Attempt to load script: pid={process_id}, resume={resume}")
 
-        session = self._attach_session(pid)
+        session = self._attach_session(process_id, process_name)
         self._unload_script(session)
 
-        kw = {}
+        kwargs = {}
         if utils.parse_version(frida.__version__) < (14,):
-            kw["runtime"] = "v8"
-        script = session.create_script(self._internal_script.source, **kw)
-        script = FridaScript(session, script)
+            kwargs["runtime"] = "v8"
+        script = FridaScript(
+            session,
+            session.create_script(self._internal_script.load(), **kwargs)
+        )
         self._manager.add_script_handler(script, self)
 
         try:
             script.load()
-            script.exports_sync.load_scripts(self._load_script_files(), self._user_parameters)
+            script.exports_sync.load_scripts(
+                self._load_script_files(),
+                self._user_parameters
+            )
         finally:
             if resume:
-                utils.ignore_error(self.device.resume, args=(pid,))
+                utils.ignore_errors(self.device.resume, args=(process_id,))
 
         self._reactor.schedule(lambda: self.on_script_loaded(script))
 
-    def _attach_session(self, pid: int):
-        session = self._manager.get_session(pid)
+    def _attach_session(self, process_id: int, process_name: str = None):
+        session = self._manager.get_session(process_id)
         if session:
             return session
 
-        _logger.debug(f"Attempt to attach process: pid={pid}")
+        _logger.debug(f"Attempt to attach process: pid={process_id}")
 
-        target_process = None
-        for process in self.device.enumerate_processes():
-            if process.pid == pid:
-                target_process = process
-        if target_process is None:
-            raise frida.ProcessNotFoundError(f"unable to find process with pid '{pid}'")
+        if not process_name:
+            for process in self.device.enumerate_processes():
+                if process.pid == process_id:
+                    process_name = process.name
+                    break
+            else:
+                raise frida.ProcessNotFoundError(f"unable to find process with pid '{process_id}'")
 
-        session = self.device.attach(target_process.pid)
-        session = FridaSession(session, target_process.name)
+        session = self.device.attach(process_id)
+        session = FridaSession(session, process_name)
         self._manager.set_session(session)
 
         if self._enable_child_gating:
-            _logger.debug(f"Enable child gating: {pid}")
-            session.enable_child_gating()
+            try:
+                session.enable_child_gating()
+            except frida.NotSupportedError:
+                _logger.warning(f"Enable child gating is not supported, ignore")
+        else:
+            try:
+                session.disable_child_gating()
+            except frida.NotSupportedError:
+                pass
 
         self._manager.add_session_handler(session, self)
         self._reactor.schedule(lambda: self.on_session_attached(session))
@@ -729,7 +835,7 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
     def _detach_session(self, session: FridaSession):
         if session is not None:
             _logger.debug(f"{session} detach")
-            utils.ignore_error(session.detach)
+            utils.ignore_errors(session.detach)
 
     def _unload_script(self, session: FridaSession):
         if not session:
@@ -739,7 +845,7 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
             if not script:
                 break
             _logger.debug(f"{script} unload")
-            utils.ignore_error(script.unload)
+            utils.ignore_errors(script.unload)
 
     def _eternalize_script(self, session: FridaSession):
         if not session:
@@ -749,7 +855,7 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
             if not script:
                 break
             _logger.debug(f"{script} eternalize")
-            utils.ignore_error(script.eternalize)
+            utils.ignore_errors(script.eternalize)
 
     def _on_stop(self):
         process_script = self._unload_script
@@ -772,7 +878,7 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
     def on_error(self, exc, traceback):
         if isinstance(exc, (KeyboardInterrupt, frida.TransportError, frida.ServerNotRunningError)):
             _logger.error(f"{traceback if environ.debug else exc}")
-            self.stop()
+            self.signal_stop()
         elif isinstance(exc, (frida.core.RPCException,)):
             _logger.error(f"{exc}")
         else:
@@ -784,7 +890,7 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
 
     def on_device_lost(self):
         _logger.info("Device lost")
-        self.stop()
+        self.signal_stop()
 
     def on_file_change(self, file: FridaScriptFile):
         """
@@ -795,6 +901,13 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
         for session in self.sessions.values():
             self.load_script(session.pid)
 
+    def _resume(self, process_id: int, process_name: str):
+        try:
+            # 可能ios设备可能会出错: https://github.com/frida/frida-core/issues/462
+            self.device.resume(process_id)
+        except Exception as e:
+            _logger.warning(f"Resume process `{process_name}` with pid {process_id} failed: {e}")
+
     def on_spawn_added(self, spawn: "_frida.Spawn"):
         """
         spaw进程添加回调，默认resume所有spawn进程
@@ -802,14 +915,12 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
         """
         _logger.debug(f"{spawn} added")
 
-        for identifier in self._target_identifiers:
-            if identifier.search(spawn.identifier):
-                self.load_script(spawn.pid, resume=True)
-                return
-        try:
-            self.device.resume(spawn.pid)
-        except Exception as e:
-            _logger.error(f"{e}")
+        if spawn and spawn.identifier:
+            for identifier in self._target_identifiers:
+                if identifier.search(spawn.identifier):
+                    self.load_script(spawn.pid, spawn.identifier, resume=True)
+                    return
+        self._reactor.schedule(lambda: self._resume(spawn.pid, spawn.identifier))
 
     def on_child_added(self, child: "_frida.Child"):
         """
@@ -817,7 +928,13 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
         :param child: 子进程信息
         """
         _logger.debug(f"{child} added")
-        self.device.resume(child.pid)
+
+        if child and child.identifier:
+            for identifier in self._target_identifiers:
+                if identifier.search(child.identifier):
+                    self.load_script(child.pid, child.identifier, resume=True)
+                    return
+        self._reactor.schedule(lambda: self._resume(child.pid, child.identifier))
 
     def on_script_loaded(self, script: FridaScript):
         """
@@ -826,24 +943,28 @@ class FridaApplication(FridaDeviceHandler, FridaSessionHandler, FridaScriptHandl
         """
         _logger.debug(f"{script} loaded")
 
-    def on_script_event(self, script: FridaScript, message: Any, data: Any):
+    def on_script_event(self, script: FridaScript, event: Any, data: Any):
         """
         脚本发送事件回调
         :param script: frida的脚本
-        :param message: 事件消息
+        :param event: 事件消息
         :param data: 事件数据
         """
-        group = Counter.Group(accept_empty=False)
-        count = self._event_counter.increase(
-            group.add(
+        pid_count = self.counter.increase(
+            FridaEventCounter.Group(accept_empty=False).add(
                 pid=script.session.pid,
-                method=utils.get_item(message, "method_name"),
             )
         )
-
+        pid_method_count = self.counter.increase(
+            FridaEventCounter.Group(accept_empty=False).add(
+                pid=script.session.pid,
+                method=utils.get_item(event, "method_name"),
+            )
+        )
+        # 日志展示当前进程当前方法一共出现的次数和具体详情
         _logger.info(
-            f"{script} event count={count} in the {group}: {os.linesep}"
-            f"{json.dumps(message, indent=2, ensure_ascii=False)}",
+            f"{script} event pid@index={pid_count} pid+method@index={pid_method_count} {os.linesep}"
+            f"{json.dumps(event, indent=2, ensure_ascii=False)}",
         )
 
     def on_script_send(self, script: FridaScript, payload: Any, data: Any):
