@@ -22,8 +22,8 @@
       +-----------------------+  ;;  | |         |,"
          /_)______________(_/  //'   | +---------+
     ___________________________/___  `,
-   /  oooooooooooooooo  .o.  oooo /,   \,"-----------
-  / ==ooooooooooooooo==.o.  ooo= //   ,`\--{)B     ,"
+   /  oooooooooooooooo  .o.  oooo /,   `,"-----------
+  / ==ooooooooooooooo==.o.  ooo= //   ,``--{)B     ,"
  /_==__==========__==_ooo__ooo=_/'   /___________,"
 """
 import abc
@@ -32,35 +32,31 @@ import os
 import threading
 from typing import Union, Optional
 
-from rich import get_console
-from rich.prompt import Confirm
-from rich.table import Table
-
 from .. import utils, environ
+from ..metadata import __missing__
+from ..rich import confirm
 
 _logger = environ.get_logger("frida.app")
 
 
 class FridaUserScript(metaclass=abc.ABCMeta):
-    __missing__ = object()
 
     def __init__(self):
-        self._source: Union[str, object] = self.__missing__
+        self._source: Union[str, object] = __missing__
         self._lock = threading.RLock()
 
-    @property
-    def source(self) -> Optional[str]:
-        return self.load()
-
     def load(self) -> Optional[str]:
-        with self._lock:
-            if self._source is self.__missing__:
-                self._source = self._load()
-            return self._source
+        source = self._source
+        if source is __missing__:
+            with self._lock:
+                source = self._source
+                if source is __missing__:
+                    source = self._source = self._load()
+        return source
 
     def clear(self) -> None:
         with self._lock:
-            self._source = self.__missing__
+            self._source = __missing__
 
     @property
     @abc.abstractmethod
@@ -72,7 +68,7 @@ class FridaUserScript(metaclass=abc.ABCMeta):
         pass
 
     def as_dict(self) -> dict:
-        return {"filename": self.filename, "source": self.source}
+        return {"filename": str(self.filename), "source": self.load()}
 
     def as_json(self) -> str:
         return json.dumps(self.as_dict())
@@ -99,9 +95,8 @@ class FridaScriptFile(FridaUserScript):
         return self._path
 
     def _load(self) -> Optional[str]:
-        with open(self._path, "rb") as f:
-            _logger.info(f"Load {self}")
-            return f.read().decode("utf-8")
+        _logger.info(f"Load {self}")
+        return utils.read_file(self._path, text=True)
 
 
 class FridaEvalCode(FridaUserScript):
@@ -131,27 +126,30 @@ class FridaShareScript(FridaUserScript):
         return self._url
 
     def _load(self):
+        with environ.get_url_file(self._url) as file:  # 文件锁，避免多进程同时操作
 
-        with utils.UrlFile(self._url) as file:  # 文件锁，避免多进程同时操作
+            # 如果是本地文件，直接就返回了
+            if file.is_local:
+                _logger.info(f"Load local {self}")
+                source = utils.read_file(file.save(), text=True)
+                return source
 
+            # 判断是否需要情况缓存
             if not self._cached:
                 file.clear()
 
             _logger.info(f"Download {self}")
-            target_path = file.save()
-
-            with open(target_path, "rb") as f:
-                source = f.read().decode("utf-8")
+            dest_path = file.save()
 
+            source = utils.read_file(dest_path, text=True)
             if self._trusted:
                 _logger.info(f"Load trusted {self}")
                 return source
 
             cached_md5 = ""
-            cached_md5_path = target_path + ".md5"
+            cached_md5_path = dest_path + ".md5"
             if os.path.exists(cached_md5_path):
-                with open(cached_md5_path, "rt") as fd:
-                    cached_md5 = fd.read()
+                cached_md5 = utils.read_file(cached_md5_path, text=True)
 
             source_md5 = utils.get_md5(source)
             if cached_md5 == source_md5:
@@ -164,23 +162,16 @@ class FridaShareScript(FridaUserScript):
             if len(source_lines) > line_count:
                 source_summary += "... ..."
 
-            table = Table(show_lines=True)
-            table.add_column("Property", justify="right", style="cyan", no_wrap=True)
-            table.add_column("Content", style="magenta")
-            table.add_row("Url", self._url)
-            table.add_row("Original md5", cached_md5)
-            table.add_row("Current md5", source_md5)
-            table.add_row("Source", source_summary)
-
-            console = get_console()
-            console.print(table)
-
             prompt = f"This is the first time you're running this particular snippet, " \
-                     f"or the snippet's source code has changed. {os.linesep}" \
+                     f"or the snippet's source code has changed.{os.linesep}" \
+                     f"Url: {self._url}{os.linesep}" \
+                     f"Original md5: {cached_md5}{os.linesep}" \
+                     f"Current md5: {source_md5}{os.linesep}" \
+                     f"Source: {os.linesep}{source_summary}{os.linesep}" \
                      f"Are you sure you'd like to trust it?"
-            if Confirm.ask(prompt, console=console):
-                with open(cached_md5_path, "wt") as fd:
-                    fd.write(source_md5)
+
+            if confirm(prompt):
+                utils.write_file(cached_md5_path, source_md5)
                 _logger.info(f"Load trusted {self}")
                 return source
             else:
