@@ -15,17 +15,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
+import typing
+
+from ..types import NoFreePortFoundError
+
+if typing.TYPE_CHECKING:
+    import socket
+
 
 # from: https://github.com/google/python_portpicker
 
-import socket
-
-
-class NoFreePortFoundError(Exception):
-    """Exception indicating that no free port could be found."""
-
-
-def bind(port: int, socket_type: socket.SocketKind, socket_proto: int):
+def bind(port: int, socket_type: "socket.SocketKind", socket_proto: int):
     """Try to bind to a socket of the specified type, protocol, and port.
 
     This is primarily a helper function for PickUnusedPort, used to see
@@ -43,6 +43,8 @@ def bind(port: int, socket_type: socket.SocketKind, socket_proto: int):
     Returns:
       The port number on success or None on failure.
     """
+    import socket
+
     got_socket = False
     for family in (socket.AF_INET6, socket.AF_INET):
         try:
@@ -52,7 +54,7 @@ def bind(port: int, socket_type: socket.SocketKind, socket_proto: int):
             continue
         try:
             # sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-            sock.bind(('', port))
+            sock.bind(('0.0.0.0', port))
             if socket_type == socket.SOCK_STREAM:
                 sock.listen(1)
             port = sock.getsockname()[1]
@@ -71,12 +73,27 @@ def is_port_free(port: int):
     Returns:
       boolean, whether it is free to use for both TCP and UDP
     """
+    import socket
+
     return bind(port, socket.SOCK_STREAM, socket.IPPROTO_TCP) is not None and \
-           bind(port, socket.SOCK_DGRAM, socket.IPPROTO_UDP) is not None
+        bind(port, socket.SOCK_DGRAM, socket.IPPROTO_UDP) is not None
 
 
-def pick_unused_port(ports: [int] = range(47134, 52134)):
-    for port in ports:
-        if is_port_free(port):
-            return port
-    raise NoFreePortFoundError()
+def get_free_port():
+    import socket
+
+    try:
+        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        s.bind(('127.0.0.1', 0))
+        try:
+            return s.getsockname()[1]
+        finally:
+            s.close()
+    except OSError:
+        import random
+
+        for _ in range(20):
+            port = random.randint(30000, 40000)
+            if not is_port_free(port):
+                return port
+        raise NoFreePortFoundError("No free port found")
